@@ -2,7 +2,7 @@
 
 /*
  * Copyright (c) 2009, Sun Microsystems, Inc.
- * Copyright (c) 2012-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,9 @@
 #if defined(HAVE_BLKIN)
 #include <blkin/zipkin_c.h>
 #endif
+#ifdef USE_LTTNG_NTIRPC
+#include "lttng/xprt.h"
+#endif
 
 typedef struct svc_xprt SVCXPRT;
 
@@ -126,6 +129,7 @@ typedef struct svc_init_params {
 	u_int max_events;	/* evchan events */
 	u_int ioq_send_max;
 	u_int ioq_thrd_max;
+	u_int ioq_thrd_min;
 	u_int gss_ctx_hash_partitions;
 	u_int gss_max_ctx;
 	u_int gss_max_idle_gen;
@@ -260,7 +264,7 @@ struct svc_xprt {
 	int xp_si_type;		/* si type */
 	int xp_type;		/* xprt type */
 
-	uint32_t xp_refs;	/* handle reference count */
+	int32_t xp_refcnt;	/* handle reference count */
 	uint16_t xp_flags;	/* flags */
 };
 
@@ -292,8 +296,8 @@ struct svc_req {
 	SVCXPRT *rq_xprt;	/* associated transport */
 
 	/* New with TI-RPC */
-	caddr_t rq_clntname;	/* read only client name */
-	caddr_t rq_svcname;	/* read only cooked service cred */
+	char *rq_clntname;	/* read only client name */
+	char *rq_svcname;	/* read only cooked service cred */
 
 	/* New with N TI-RPC */
 	XDR *rq_xdrs;
@@ -313,7 +317,7 @@ struct svc_req {
 	/* blkin tracing */
 	struct blkin_trace bl_trace;
 #endif
-	uint32_t rq_refs;
+	uint32_t rq_refcnt;
 };
 
 /*
@@ -368,28 +372,38 @@ __END_DECLS
 	if (((req)->rq_xprt)->xp_ops->xp_checksum) \
 		(*((req)->rq_xprt)->xp_ops->xp_checksum)(req, what, length)
 
-/* Protect a SVCXPRT with a SVC_REF for each call or request.
+/* Protect a SVCXPRT with SVC_REF() for each call, request, or task thread.
  */
 static inline void svc_ref_it(SVCXPRT *xprt, u_int flags,
 			      const char *tag, const int line)
 {
-	atomic_inc_uint32_t(&xprt->xp_refs);
+#ifdef USE_LTTNG_NTIRPC
+	int32_t refs =
+#endif /* USE_LTTNG_NTIRPC */
+		atomic_inc_int32_t(&xprt->xp_refcnt);
 
 	if (flags & SVC_REF_FLAG_LOCKED)  {
 		/* unlock before warning trace */
 		mutex_unlock(&xprt->xp_lock);
 	}
 	XPRT_TRACE(xprt, __func__, tag, line);
+#ifdef USE_LTTNG_NTIRPC
+	tracepoint(xprt, ref, tag, line, xprt, refs);
+#endif /* USE_LTTNG_NTIRPC */
 }
 #define SVC_REF2(xprt, flags, tag, line)				\
 	svc_ref_it(xprt, flags, tag, line)
 #define SVC_REF(xprt, flags)						\
 	svc_ref_it(xprt, flags, __func__, __LINE__)
 
+/* SVC_RELEASE() the SVC_REF().
+ * Idempotent SVC_XPRT_FLAG_DESTROYED (bit SVC_XPRT_FLAG_RELEASING)
+ * indicates that more references should not be taken.
+ */
 static inline void svc_release_it(SVCXPRT *xprt, u_int flags,
 				  const char *tag, const int line)
 {
-	uint32_t refs = atomic_dec_uint32_t(&xprt->xp_refs);
+	int32_t refs = atomic_dec_int32_t(&xprt->xp_refcnt);
 	uint16_t xp_flags;
 
 	if (flags & SVC_RELEASE_FLAG_LOCKED) {
@@ -397,6 +411,9 @@ static inline void svc_release_it(SVCXPRT *xprt, u_int flags,
 		mutex_unlock(&xprt->xp_lock);
 	}
 	XPRT_TRACE(xprt, __func__, tag, line);
+#ifdef USE_LTTNG_NTIRPC
+	tracepoint(xprt, unref, tag, line, xprt, refs);
+#endif /* USE_LTTNG_NTIRPC */
 
 	if (likely(refs > 0)) {
 		/* normal case */
@@ -420,8 +437,9 @@ static inline void svc_release_it(SVCXPRT *xprt, u_int flags,
 #define SVC_RELEASE(xprt, flags)					\
 	svc_release_it(xprt, flags, __func__, __LINE__)
 
-/* SVC_DESTROY is SVC_RELEASE with once-only semantics.  Also, idempotent
- * SVC_XPRT_FLAG_DESTROYED indicates that more references should not be taken.
+/* SVC_DESTROY() is SVC_RELEASE() with once-only semantics.
+ * Idempotent SVC_XPRT_FLAG_DESTROYED (bit SVC_XPRT_FLAG_DESTROYING)
+ * indicates that more references should not be taken.
  */
 static inline void svc_destroy_it(SVCXPRT *xprt,
 				  const char *tag, const int line)
