@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2010, Oracle America, Inc.
- * Copyright (c) 2012-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <config.h>
+#include "config.h"
 #include <pthread.h>
 #include <reentrant.h>
 #include <sys/types.h>
@@ -314,6 +314,7 @@ clnt_tli_ncreate(int fd, const struct netconfig *nconf,
 	int save_errno;
 	int one = 1;
 	uint32_t flags = CLNT_CREATE_FLAG_CONNECT;
+	struct timeval timeval;
 
 	if (fd == RPC_ANYFD) {
 		if (nconf == NULL) {
@@ -358,6 +359,18 @@ clnt_tli_ncreate(int fd, const struct netconfig *nconf,
 
 	switch (servtype) {
 	case NC_TPI_COTS:
+		if (nconf && ((strcmp(nconf->nc_protofmly, "inet") == 0)
+			      || (strcmp(nconf->nc_protofmly, "inet6") == 0))) {
+			/* set SO_SNDTIMEO to deal with bad clients */
+			timeval.tv_sec = 5;
+			timeval.tv_usec = 0;
+			if (setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO,
+				       (char *)&timeval, sizeof(timeval))) {
+				__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
+					"%s: fd %d SO_SNDTIMEO failed (%d)",
+					__func__, fd, errno);
+			}
+		}
 		cl = clnt_vc_ncreatef(fd, svcaddr, prog, vers, sendsz, recvsz,
 				      flags);
 		break;
@@ -366,6 +379,16 @@ clnt_tli_ncreate(int fd, const struct netconfig *nconf,
 			      || (strcmp(nconf->nc_protofmly, "inet6") == 0))) {
 			(void) setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &one,
 					  sizeof(one));
+			/* set SO_SNDTIMEO to deal with bad clients */
+			timeval.tv_sec = 5;
+			timeval.tv_usec = 0;
+			if (setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO,
+				       (char *)&timeval, sizeof(timeval))) {
+				__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
+					"%s: fd %d SO_SNDTIMEO failed (%d)",
+					__func__, fd, errno);
+			}
+
 		}
 		cl = clnt_vc_ncreatef(fd, svcaddr, prog, vers, sendsz, recvsz,
 				      flags);
@@ -665,15 +688,19 @@ clnt_req_wait_reply(struct clnt_req *cc)
 int
 clnt_req_release(struct clnt_req *cc)
 {
-	uint32_t refs = atomic_dec_uint32_t(&cc->cc_refs);
+	int refs = atomic_dec_int32_t(&cc->cc_refcnt);
 
-	if (refs)
+	if (likely(refs > 0)) {
+		/* normal case */
 		return (refs);
+	}
 
 	clnt_req_reset(cc);
 	clnt_req_fini(cc);
+	CLNT_RELEASE(cc->cc_clnt, CLNT_RELEASE_FLAG_NONE);
+
 	(*cc->cc_free_cb)(cc, cc->cc_size);
-	return (0);
+	return (refs);
 }
 
 /*
