@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2010, Oracle America, Inc.
- * Copyright (c) 2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,7 @@
  * rpcb_clnt.c
  * interface to rpcbind rpc service.
  */
-#include <config.h>
+#include "config.h"
 #include <pthread.h>
 #include <reentrant.h>
 #include <sys/stat.h>
@@ -40,6 +40,7 @@
 #include <sys/un.h>
 #include <sys/utsname.h>
 #include <rpc/rpc.h>
+#include <rpc/xdr_inline.h>
 #include <rpc/rpcb_prot.h>
 #include <rpc/nettype.h>
 #include <netconfig.h>
@@ -295,7 +296,6 @@ static CLIENT *getclnthandle(const char *host, const struct netconfig *nconf,
 		t = rpc_sperror(&client->cl_error, __func__);
 		__warnx(TIRPC_DEBUG_FLAG_CLNT_RPCB, "%s", t);
 		mem_free(t, RPC_SPERROR_BUFLEN);
-		goto out_err;
 
 		addr_to_delete.len = addr->len;
 		addr_to_delete.buf = (char *)mem_zalloc(addr->len);
@@ -646,6 +646,8 @@ __rpcb_findaddr_timed(rpcprog_t program, rpcvers_t version,
 	rpcvers_t start_vers = RPCBVERS4;
 	enum clnt_stat clnt_st;
 
+	parms.r_addr = NULL;
+
 	/* parameter checking */
 	if (nconf == NULL) {
 		__warnx(TIRPC_DEBUG_FLAG_ERROR, "%s: %s",
@@ -655,8 +657,6 @@ __rpcb_findaddr_timed(rpcprog_t program, rpcvers_t version,
 		goto done;
 	}
 
-	parms.r_addr = NULL;
-
 	/* authnone handle */
 	auth = authnone_ncreate();	/* idempotent */
 
@@ -676,7 +676,7 @@ __rpcb_findaddr_timed(rpcprog_t program, rpcvers_t version,
 		struct netbuf remote;
 		struct pmap pmapparms;
 		rpcvers_t pmapvers = 2;
-		u_short port = 0;
+		uint16_t port = 0;
 
 		/*
 		 * Try UDP only - there are some portmappers out
@@ -711,7 +711,7 @@ __rpcb_findaddr_timed(rpcprog_t program, rpcvers_t version,
 		cc = mem_alloc(sizeof(*cc));
 		clnt_req_fill(cc, client, auth, PMAPPROC_GETPORT,
 			      (xdrproc_t) xdr_pmap, &pmapparms,
-			      (xdrproc_t) xdr_u_short, &port);
+			      (xdrproc_t) xdr_uint16_t, &port);
 		clnt_st = clnt_req_setup(cc, tv);
 		if (clnt_st == RPC_SUCCESS) {
 			clnt_st = CLNT_CALL_WAIT(cc);
@@ -740,8 +740,8 @@ __rpcb_findaddr_timed(rpcprog_t program, rpcvers_t version,
 		address->buf = (char *)mem_alloc(remote.len);
 
 		memcpy(address->buf, remote.buf, remote.len);
-		memcpy(&((char *)address->buf)[sizeof(short)],
-		       (char *)(void *)&port, sizeof(short));
+		memcpy(&((char *)address->buf)[sizeof(uint16_t)],
+		       (char *)(void *)&port, sizeof(uint16_t));
 		address->len = address->maxlen = remote.len;
 		goto done;
 	}
@@ -847,8 +847,9 @@ __rpcb_findaddr_timed(rpcprog_t program, rpcvers_t version,
 		xdr_free((xdrproc_t) xdr_wrapstring, (char *)(void *)&ua);
 	if (clpp)
 		*clpp = client;
-	else if (client)
+	else
 		CLNT_DESTROY(client);
+
 	if (parms.r_addr != NULL && parms.r_addr != nullstring)
 		mem_free(parms.r_addr, 0);
 	return (address);
@@ -983,11 +984,11 @@ rpcb_getmaps(const struct netconfig *nconf, const char *host)
 enum clnt_stat
 rpcb_rmtcall(const struct netconfig *nconf, /* Netconfig structure */
 	     const char *host, /* Remote host name */
-	     rpcprog_t prog, rpcvers_t vers,
-	     rpcproc_t proc,/* Remote proc identifiers */
-	     xdrproc_t xdrargs, caddr_t argsp, xdrproc_t xdrres,
-	     caddr_t resp,	/* Argument and Result */
-	     struct timeval tout,	/* Timeout value for this call */
+	     const rpcprog_t prog, const rpcvers_t vers,
+	     const rpcproc_t proc,/* Remote proc identifiers */
+	     const xdrproc_t xdrargs, void *argsp,
+	     const xdrproc_t xdrres, void *resp, /* Argument and Result */
+	     const struct timeval tout,	/* Timeout value for this call */
 	     const struct netbuf *addr_ptr
 	     /* Preallocated netbuf address */)
 {
@@ -1087,6 +1088,7 @@ boolrpcb_gettime(const char *host, time_t *timep)
 	void *handle;
 	struct netconfig *nconf;
 	rpcvers_t vers;
+	int32_t time32 = 0; 	/* old protocol 32-bits not long time_t */
 	enum clnt_stat st;
 
 	if ((host == NULL) || (host[0] == 0)) {
@@ -1115,6 +1117,11 @@ boolrpcb_gettime(const char *host, time_t *timep)
 	} while (CLNT_FAILURE(client));
 
 	__rpc_endconf(handle);
+
+	if (!client) {
+		return false;
+	}
+
 	if (CLNT_FAILURE(client)) {
 		CLNT_DESTROY(client);
 		return (false);
@@ -1123,7 +1130,7 @@ boolrpcb_gettime(const char *host, time_t *timep)
 	cc = mem_alloc(sizeof(*cc));
 	clnt_req_fill(cc, client, authnone_ncreate(), RPCBPROC_GETTIME,
 		      (xdrproc_t) xdr_void, NULL,
-		      (xdrproc_t) xdr_int, timep);
+		      (xdrproc_t) xdr_int32_t, &time32);
 	st = clnt_req_setup(cc, to);
 	if (st == RPC_SUCCESS) {
 		st = CLNT_CALL_WAIT(cc);
@@ -1153,6 +1160,7 @@ boolrpcb_gettime(const char *host, time_t *timep)
 
 	clnt_req_release(cc);
 	CLNT_DESTROY(client);
+	*timep = time32;
 	return (st == RPC_SUCCESS ? true : false);
 }
 
@@ -1270,7 +1278,7 @@ static CLIENT *local_rpcb(const char *tag)
 {
 	CLIENT *client = NULL;
 	char *t;
-	static struct netconfig *loopnconf;
+	static struct netconfig *loopnconf = NULL;
 	static char *hostname;
 	extern mutex_t loopnconf_lock;
 	struct netbuf nbuf;
@@ -1285,8 +1293,11 @@ static CLIENT *local_rpcb(const char *tag)
 	 */
 	memset(&sun, 0, sizeof(sun));
 	sock = socket(AF_LOCAL, SOCK_STREAM, 0);
-	if (sock < 0)
+	if (sock < 0) {
+		/* For error codes */
+		client = clnt_raw_ncreate(RPCBPROG, RPCBVERS);
 		goto try_nconf;
+	}
 	sun.sun_family = AF_LOCAL;
 	strcpy(sun.sun_path, _PATH_RPCBINDSOCK);
 	nbuf.len = SUN_LEN(&sun);
