@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2009, Sun Microsystems, Inc.
- * Copyright (c) 2012-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <config.h>
+#include "config.h"
 
 /*
  * svc_vc.c, Server side for Connection Oriented RPC.
@@ -248,7 +248,7 @@ svc_vc_ncreatef(const int fd, const u_int sendsz, const u_int recvsz,
 	     && !(flags & SVC_CREATE_FLAG_XPRT_NOREG))
 	    || (flags & SVC_CREATE_FLAG_XPRT_DOREG))
 		svc_rqst_evchan_reg(__svc_params->ev_u.evchan.id, xprt,
-				    SVC_RQST_FLAG_LOCKED |
+				    RPC_DPLX_LOCKED |
 				    SVC_RQST_FLAG_CHAN_AFFINITY);
 
 	/* release */
@@ -276,13 +276,6 @@ makefd_xprt(const int fd, const u_int sendsz, const u_int recvsz,
 
 	assert(fd != -1);
 
-	if (!svc_vc_new_conn_ok()) {
-		__warnx(TIRPC_DEBUG_FLAG_ERROR,
-			"%s: fd %d max_connections exceeded\n",
-			__func__, fd);
-		return (NULL);
-	}
-
 	/* atomically find or create shared fd state; ref+1; locked */
 	xprt = svc_xprt_lookup(fd, svc_vc_xprt_setup);
 	if (!xprt) {
@@ -419,6 +412,7 @@ svc_vc_rendezvous(SVCXPRT *xprt)
 	int rc;
 	socklen_t len;
 	static int n = 1;
+	struct timeval timeval;
 
  again:
 	len = sizeof(addr);
@@ -448,6 +442,7 @@ svc_vc_rendezvous(SVCXPRT *xprt)
 		__warnx(TIRPC_DEBUG_FLAG_ERROR,
 			"%s: %p fd %d svc_rqst_rearm_events failed (will set dead)",
 			__func__, xprt, xprt->xp_fd);
+		close(fd);
 		return (XPRT_DIED);
 	}
 
@@ -475,6 +470,16 @@ svc_vc_rendezvous(SVCXPRT *xprt)
 				  sizeof(len));
 	}
 
+	/* set SO_SNDTIMEO to deal with bad clients */
+	timeval.tv_sec = 5;
+	timeval.tv_usec = 0;
+	if (setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeval,
+		       sizeof(timeval))) {
+		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
+			"%s: fd %d SO_SNDTIMEO failed (%d)",
+			 __func__, fd, errno);
+	}
+
 	__rpc_address_setup(&newxprt->xp_local);
 	rc = getsockname(fd, newxprt->xp_local.nb.buf,
 			 &newxprt->xp_local.nb.len);
@@ -515,10 +520,10 @@ svc_vc_destroy_task(struct work_pool_entry *wpe)
 	uint16_t xp_flags;
 
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-		"%s() %p fd %d xp_refs %" PRIu32,
-		__func__, rec, rec->xprt.xp_fd, rec->xprt.xp_refs);
+		"%s() %p fd %d xp_refcnt %" PRId32,
+		__func__, rec, rec->xprt.xp_fd, rec->xprt.xp_refcnt);
 
-	if (rec->xprt.xp_refs) {
+	if (rec->xprt.xp_refcnt) {
 		/* instead of nanosleep */
 		work_pool_submit(&svc_work_pool, &(rec->ioq.ioq_wpe));
 		return;
@@ -554,13 +559,11 @@ svc_vc_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 		.tv_nsec = 0,
 	};
 
-	/* clears xprt from the xprt table (eg, idle scans) */
-	svc_rqst_xprt_unregister(xprt);
+	svc_rqst_xprt_unregister(xprt, flags);
 
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-		"%s() %p fd %d xp_refs %" PRIu32
-		" should actually destroy things @ %s:%d",
-		__func__, xprt, xprt->xp_fd, xprt->xp_refs, tag, line);
+		"%s() %p fd %d xp_refcnt %" PRId32 " @%s:%d",
+		__func__, xprt, xprt->xp_fd, xprt->xp_refcnt, tag, line);
 
 	while (atomic_postset_uint16_t_bits(&(REC_XPRT(xprt)->ioq.ioq_s.qflags),
 					    IOQ_FLAG_WORKING)
@@ -575,9 +578,6 @@ svc_vc_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 static void
 svc_vc_destroy(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 {
-	/* connection tracking--decrement now, he's dead jim */
-	svc_vc_dec_nconns();
-
 	svc_vc_destroy_it(xprt, flags, tag, line);
 }
 
