@@ -1,7 +1,6 @@
 /*
  * Copyright (c) 2012 Linux Box Corporation.
- * Copyright (c) 2013-2015 CohortFS, LLC.
- * Copyright (c) 2017 Red Hat, Inc.
+ * Copyright (c) 2013-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +24,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 #include <sys/poll.h>
@@ -39,7 +38,7 @@
 #include <misc/portable.h>
 #include <rpc/rpc.h>
 #include "rpc_com.h"
-#include "rpc_dplx_internal.h"
+#include "svc_internal.h"
 #include "svc_xprt.h"
 
 /**
@@ -67,6 +66,7 @@ static bool initialized;
 struct svc_xprt_fd {
 	mutex_t lock;
 	struct rbtree_x xt;
+	uint32_t connections;
 };
 
 static struct svc_xprt_fd svc_xprt_fd = {
@@ -132,7 +132,7 @@ svc_xprt_init_failure(void)
 }
 
 /*
- * On success, returns with RPC_DPLX_FLAG_LOCKED
+ * On success, returns with RPC_DPLX_LOCKED
  */
 SVCXPRT *
 svc_xprt_lookup(int fd, svc_xprt_setup_t setup)
@@ -160,7 +160,17 @@ svc_xprt_lookup(int fd, svc_xprt_setup_t setup)
 		rwlock_wrlock(&t->lock);
 		nv = opr_rbtree_lookup(&t->t, &sk.fd_node);
 		if (!nv) {
-			(*setup)(&xprt); /* zalloc, xp_refs = 1 */
+			if (atomic_inc_uint32_t(&svc_xprt_fd.connections)
+			    > __svc_params->max_connections) {
+				atomic_dec_uint32_t(&svc_xprt_fd.connections);
+				rwlock_unlock(&t->lock);
+				__warnx(TIRPC_DEBUG_FLAG_ERROR,
+					"%s: fd %d max_connections %u exceeded\n",
+					__func__, fd,
+					__svc_params->max_connections);
+				return (NULL);
+			}
+			(*setup)(&xprt); /* zalloc, xp_refcnt = 1 */
 			xprt->xp_fd = fd;
 			xprt->xp_flags = SVC_XPRT_FLAG_INITIAL;
 
@@ -173,6 +183,7 @@ svc_xprt_lookup(int fd, svc_xprt_setup_t setup)
 					"%s: collision inserting in locked rbtree partition",
 					__func__);
 				(*setup)(&xprt);	/* free, sets NULL */
+				atomic_dec_uint32_t(&svc_xprt_fd.connections);
 			}
 			rwlock_unlock(&t->lock);
 			return (xprt);
@@ -227,6 +238,7 @@ svc_xprt_clear(SVCXPRT *xprt)
 		/* if another thread passes test during svc_xprt_shutdown(),
 		 * this lock (and generation test) prevents repeats.
 		 */
+		atomic_dec_uint32_t(&svc_xprt_fd.connections);
 		rwlock_wrlock(&t->lock);
 		opr_rbtree_remove(&t->t, &REC_XPRT(xprt)->fd_node);
 		rwlock_unlock(&t->lock);
@@ -354,7 +366,7 @@ svc_xprt_shutdown()
 			/* prevent repeats, see svc_xprt_clear() */
 			opr_rbtree_remove(&t->t, &rec->fd_node);
 
-			/* fd_node is counted by initial xp_refs = 1,
+			/* fd_node is counted by initial xp_refcnt = 1,
 			 * SVC_DESTROY() decrements that reference.
 			 */
 			rwlock_unlock(&t->lock);
@@ -375,9 +387,9 @@ void
 svc_xprt_trace(SVCXPRT *xprt, const char *func, const char *tag, const int line)
 {
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-		"%s() %p fd %d xp_refs %" PRId32
-		" af %u port %u @ %s:%d",
-		func, xprt, xprt->xp_fd, xprt->xp_refs,
+		"%s() %p fd %d xp_refcnt %" PRId32
+		" af %u port %u @%s:%d",
+		func, xprt, xprt->xp_fd, xprt->xp_refcnt,
 		xprt->xp_remote.ss.ss_family,
 		__rpc_address_port(&xprt->xp_remote),
 		tag, line);
